@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003 Matteo Frigo
- * Copyright (c) 2003 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2006 Matteo Frigo
+ * Copyright (c) 2003, 2006 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,8 +22,32 @@
 #ifndef __API_H__
 #define __API_H__
 
-/* just in case: force <fftw3.h> not to use C99 complex numbers */
-#undef _Complex_I
+#ifndef CALLING_FFTW /* defined in hook.c, when calling internal functions */
+#  define COMPILING_FFTW /* used for DLL symbol exporting in fftw3.h */
+#endif
+
+/* When compiling with GNU libtool on Windows, DLL_EXPORT is #defined
+   for compiling the shared-library code.  In this case, we'll #define
+   FFTW_DLL to add dllexport attributes to the specified functions in
+   fftw3.h.
+
+   If we don't specify dllexport explicitly, then libtool
+   automatically exports all symbols.  However, if we specify
+   dllexport explicitly for any functions, then libtool apparently
+   doesn't do any automatic exporting.  (Not documented, grrr, but
+   this is the observed behavior with libtool 1.5.8.)  Thus, using
+   this forces us to correctly dllexport every exported symbol, or
+   linking bench.exe will fail.  This has the advantage of forcing
+   us to mark things correctly, which is necessary for other compilers
+   (such as MS VC++). */
+#ifdef DLL_EXPORT
+#  define FFTW_DLL
+#endif
+
+/* just in case: force <fftw3.h> not to use C99 complex numbers
+   (we need this for IBM xlc because _Complex_I is treated specially
+   and is defined even if <complex.h> is not included) */
+#define FFTW_NO_Complex
 
 #include "fftw3.h"
 #include "ifftw.h"
@@ -57,10 +81,11 @@ int X(many_kosherp)(int rnk, const int *n, int howmany);
 int X(guru_kosherp)(int rank, const X(iodim) *dims,
 		    int howmany_rank, const X(iodim) *howmany_dims);
 
+/* Note: FFTW_EXTERN is used for "internal" functions used in tests/hook.c */
 
-printer *X(mkprinter_file)(FILE *f);
+FFTW_EXTERN printer *X(mkprinter_file)(FILE *f);
 
-planner *X(the_planner)(void);
+FFTW_EXTERN planner *X(the_planner)(void);
 void X(configure_planner)(planner *plnr);
 
 void X(mapflags)(planner *, unsigned);
