@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003 Matteo Frigo
- * Copyright (c) 2003 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2006 Matteo Frigo
+ * Copyright (c) 2003, 2006 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *
  */
 
-/* $Id: dft.h,v 1.35 2003/03/15 20:29:42 stevenj Exp $ */
+/* $Id: dft.h,v 1.46 2006-01-05 03:04:26 stevenj Exp $ */
 
 #ifndef __DFT_H__
 #define __DFT_H__
@@ -33,9 +33,6 @@ typedef struct {
      R *ri, *ii, *ro, *io;
 } problem_dft;
 
-int X(problem_dft_p)(const problem *p);
-#define DFTP X(problem_dft_p)  /* shorthand */
-
 void X(dft_zerotens)(tensor *sz, R *ri, R *ii);
 problem *X(mkproblem_dft)(const tensor *sz, const tensor *vecsz,
                           R *ri, R *ii, R *ro, R *io);
@@ -60,28 +57,22 @@ plan *X(mkplan_dft)(size_t size, const plan_adt *adt, dftapply apply);
 
 /* various solvers */
 solver *X(mksolver_dft_direct)(kdft k, const kdft_desc *desc);
-solver *X(mksolver_dft_ct_dit)(kdft_dit codelet, const ct_desc *desc);
-solver *X(mksolver_dft_ct_ditbuf)(kdft_dit codelet, const ct_desc *desc);
-solver *X(mksolver_dft_ct_dif)(kdft_dif codelet, const ct_desc *desc);
-solver *X(mksolver_dft_ct_ditf)(kdft_difsq codelet, const ct_desc *desc);
-
-extern void (*X(kdft_dit_register_hook))(planner *, kdft_dit, const ct_desc *);
-extern void (*X(kdft_dif_register_hook))(planner *, kdft_dif, const ct_desc *);
+solver *X(mksolver_dft_directbuf)(kdft k, const kdft_desc *desc);
 
 void X(dft_rank0_register)(planner *p);
 void X(dft_rank_geq2_register)(planner *p);
 void X(dft_indirect_register)(planner *p);
+void X(dft_indirect_transpose_register)(planner *p);
 void X(dft_vrank_geq1_register)(planner *p);
 void X(dft_vrank2_transpose_register)(planner *p);
 void X(dft_vrank3_transpose_register)(planner *p);
 void X(dft_buffered_register)(planner *p);
 void X(dft_generic_register)(planner *p);
 void X(dft_rader_register)(planner *p);
+void X(dft_bluestein_register)(planner *p);
 void X(dft_nop_register)(planner *p);
-
-/* rader-omega.c: auxiliary stuff for rader */
-R *X(dft_rader_mkomega)(plan *p_, int n, int ginv);
-void X(dft_rader_free_omega)(R **omega);
+void X(ct_generic_register)(planner *p);
+void X(ct_genericbuf_register)(planner *p);
 
 /* configurations */
 void X(dft_conf_standard)(planner *p);
