@@ -1,7 +1,7 @@
 (*
  * Copyright (c) 1997-1999 Massachusetts Institute of Technology
- * Copyright (c) 2003 Matteo Frigo
- * Copyright (c) 2003 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2006 Matteo Frigo
+ * Copyright (c) 2003, 2006 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *)
-(* $Id: gen_twiddle.ml,v 1.16 2003/04/16 19:51:27 athena Exp $ *)
+(* $Id: gen_twiddle.ml,v 1.24 2006-02-12 23:34:12 athena Exp $ *)
 
 open Util
 open Genutil
 open C
 
-let cvsid = "$Id: gen_twiddle.ml,v 1.16 2003/04/16 19:51:27 athena Exp $"
+let cvsid = "$Id: gen_twiddle.ml,v 1.24 2006-02-12 23:34:12 athena Exp $"
 
 type ditdif = DIT | DIF
 let ditdif = ref DIT
@@ -62,10 +62,10 @@ let generate n =
 
   let sign = !Genutil.sign 
   and name = !Magic.codelet_name 
-  and byvl x = choose_simd x (ctimes (CVar "VL", x)) in
+  and byvl x = choose_simd x (ctimes (CVar "(2 * VL)", x)) in
   let ename = expand_name name in
 
-  let (bytwiddle, num_twiddles, twdesc) = Twiddle.twiddle_policy () in
+  let (bytwiddle, num_twiddles, twdesc) = Twiddle.twiddle_policy false in
   let nt = num_twiddles n in
 
   let byw = bytwiddle n sign (twiddle_array nt twarray) in
@@ -96,7 +96,7 @@ let generate n =
   let annot = standard_optimizer odag in
 
   let body = Block (
-    [Decl ("int", i)],
+    [Decl ("INT", i)],
     [For (Expr_assign (CVar i, CVar m),
 	  Binop (" > ", CVar i, Integer 0),
 	  list_to_comma 
@@ -106,7 +106,9 @@ let generate n =
 	     Expr_assign (CVar iioarray, CPlus [CVar iioarray; 
 						byvl (CVar !Simd.ivs)]);
 	     Expr_assign (CVar twarray, CPlus [CVar twarray; 
-					       byvl (Integer nt)])],
+					       byvl (Integer nt)]);
+	     make_volatile_stride (CVar iostride)
+	   ],
 	  Asch annot);
      Return (CVar twarray)
    ])
@@ -118,16 +120,16 @@ let generate n =
 	  Decl (C.realtypep, iioarray);
 	  Decl (C.constrealtypep, twarray);
 	  Decl (C.stridetype, iostride);
-	  Decl ("int", m);
-	  Decl ("int", dist)],
+	  Decl ("INT", m);
+	  Decl ("INT", dist)],
          add_constants body)
   in
   let twinstr = 
     Printf.sprintf "static const tw_instr twinstr[] = %s;\n\n" 
-      (twinstr_to_string (twdesc n))
+      (twinstr_to_string "(2 * VL)" (twdesc n))
   and desc = 
     Printf.sprintf
-      "static const ct_desc desc = {%d, \"%s\", twinstr, %s, &GENUS, %s, %s, %s};\n\n"
+      "static const ct_desc desc = {%d, \"%s\", twinstr, &GENUS, %s, %s, %s, %s};\n\n"
       n name (flops_of tree) 
       (stride_to_solverparm !uiostride) "0"
       (stride_to_solverparm !udist) 
