@@ -1,7 +1,7 @@
 (*
  * Copyright (c) 1997-1999 Massachusetts Institute of Technology
- * Copyright (c) 2003 Matteo Frigo
- * Copyright (c) 2003 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2006 Matteo Frigo
+ * Copyright (c) 2003, 2006 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *)
-(* $Id: gen_twiddle_c.ml,v 1.7 2003/04/16 19:51:27 athena Exp $ *)
+(* $Id: gen_twiddle_c.ml,v 1.14 2006-02-12 23:34:12 athena Exp $ *)
 
 open Util
 open Genutil
 open C
 
-let cvsid = "$Id: gen_twiddle_c.ml,v 1.7 2003/04/16 19:51:27 athena Exp $"
+let cvsid = "$Id: gen_twiddle_c.ml,v 1.14 2006-02-12 23:34:12 athena Exp $"
 
 type ditdif = DIT | DIF
 let ditdif = ref DIT
@@ -65,16 +65,10 @@ let generate n =
   and bytwvl x = choose_simd x (ctimes (CVar "TWVL", x)) in
   let ename = expand_name name in
 
-  let (bytwiddle, num_twiddles, twdesc) = Twiddle.twiddle_policy () in
+  let (bytwiddle, num_twiddles, twdesc) = Twiddle.twiddle_policy true in
   let nt = num_twiddles n in
 
-  let tf = if sign > 0 then
-    fun x -> Complex.times (Complex.nan Expr.CPLX) (Complex.real x)
-  else
-    fun x -> Complex.times (Complex.nan Expr.CPLXJ) (Complex.real x)
-  in
-
-  let byw = bytwiddle n sign (tf @@ (twiddle_array nt twarray)) in
+  let byw = bytwiddle n sign (twiddle_array nt twarray) in
 
   let viostride = either_stride (!uiostride) (C.SVar iostride) in
   let _ = Simd.ovs := stride_to_string "dist" !udist in
@@ -103,11 +97,10 @@ let generate n =
   let annot = standard_optimizer odag in
 
   let body = Block (
-    [Decl ("int", i);
+    [Decl ("INT", i);
      Decl (C.realtypep, rioarray)],
     [Stmt_assign (CVar rioarray,
 		  CVar (if (sign < 0) then "ri" else "ii"));
-     CVar("BEGIN_SIMD();"); (* hack *)
      For (Expr_assign (CVar i, CVar m),
 	  Binop (" > ", CVar i, Integer 0),
 	  list_to_comma 
@@ -115,9 +108,10 @@ let generate n =
 	     Expr_assign (CVar rioarray, CPlus [CVar rioarray; 
 						byvl (CVar !Simd.ivs)]);
 	     Expr_assign (CVar twarray, CPlus [CVar twarray; 
-					       bytwvl (Integer nt)])],
+					       bytwvl (Integer nt)]);
+	     make_volatile_stride (CVar iostride)
+	   ],
 	  Asch annot);
-     CVar("END_SIMD();");
      Return (CVar twarray)
    ])
   in
@@ -128,16 +122,16 @@ let generate n =
 	  Decl (C.realtypep, "ii");
 	  Decl (C.constrealtypep, twarray);
 	  Decl (C.stridetype, iostride);
-	  Decl ("int", m);
-	  Decl ("int", dist)],
+	  Decl ("INT", m);
+	  Decl ("INT", dist)],
          add_constants body)
   in
   let twinstr = 
     Printf.sprintf "static const tw_instr twinstr[] = %s;\n\n" 
-      (twinstr_to_string (twdesc n))
+      (twinstr_to_string "VL" (twdesc n))
   and desc = 
     Printf.sprintf
-      "static const ct_desc desc = {%d, %s, twinstr, %s, &GENUS, %s, %s, %s};\n\n"
+      "static const ct_desc desc = {%d, %s, twinstr, &GENUS, %s, %s, %s, %s};\n\n"
       n (stringify name) (flops_of tree) 
       (stride_to_solverparm !uiostride) "0"
       (stride_to_solverparm !udist) 
