@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003 Matteo Frigo
- * Copyright (c) 2003 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2006 Matteo Frigo
+ * Copyright (c) 2003, 2006 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,12 +22,12 @@
 
 #include "ifftw.h"
 
-int X(compute_nbuf)(int n, int vl, int nbuf, int maxbufsz)
+INT X(compute_nbuf)(INT n, INT vl, INT nbuf, INT maxbufsz)
 {
-     int i; 
+     INT i; 
 
      if (nbuf * n > maxbufsz)
-          nbuf = X(imax)((int)1, maxbufsz / n);
+          nbuf = X(imax)((INT)1, maxbufsz / n);
 
      /*
       * Look for a buffer number (not too big) that divides the
