@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003 Matteo Frigo
- * Copyright (c) 2003 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2006 Matteo Frigo
+ * Copyright (c) 2003, 2006 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *
  */
 
-/* $Id: vrank-geq1-rdft2.c,v 1.34 2003/04/15 19:03:20 stevenj Exp $ */
+/* $Id: vrank-geq1-rdft2.c,v 1.43 2006-01-27 02:10:50 athena Exp $ */
 
 
 /* Plans for handling vector transform loops.  These are *just* the
@@ -48,16 +48,16 @@ typedef struct {
      plan_rdft2 super;
 
      plan *cld;
-     int vl;
-     int ivs, ovs;
+     INT vl;
+     INT ivs, ovs;
      const S *solver;
 } P;
 
 static void apply(const plan *ego_, R *r, R *rio, R *iio)
 {
      const P *ego = (const P *) ego_;
-     int i, vl = ego->vl;
-     int ivs = ego->ivs, ovs = ego->ovs;
+     INT i, vl = ego->vl;
+     INT ivs = ego->ivs, ovs = ego->ovs;
      rdft2apply cldapply = ((plan_rdft2 *) ego->cld)->apply;
 
      for (i = 0; i < vl; ++i) {
@@ -65,10 +65,10 @@ static void apply(const plan *ego_, R *r, R *rio, R *iio)
      }
 }
 
-static void awake(plan *ego_, int flg)
+static void awake(plan *ego_, enum wakefulness wakefulness)
 {
      P *ego = (P *) ego_;
-     AWAKE(ego->cld, flg);
+     X(plan_awake)(ego->cld, wakefulness);
 }
 
 static void destroy(plan *ego_)
@@ -81,7 +81,7 @@ static void print(const plan *ego_, printer *p)
 {
      const P *ego = (const P *) ego_;
      const S *s = ego->solver;
-     p->print(p, "(rdft2-vrank>=1-x%d/%d%(%p%))",
+     p->print(p, "(rdft2-vrank>=1-x%D/%d%(%p%))",
 	      ego->vl, s->vecloop_dim, ego->cld);
 }
 
@@ -93,23 +93,22 @@ static int pickdim(const S *ego, const tensor *vecsz, int oop, int *dp)
 
 static int applicable0(const solver *ego_, const problem *p_, int *dp)
 {
-     if (RDFT2P(p_)) {
-          const S *ego = (const S *) ego_;
-          const problem_rdft2 *p = (const problem_rdft2 *) p_;
-	  if (FINITE_RNK(p->vecsz->rnk)
-	      && p->vecsz->rnk > 0
-	      && pickdim(ego, p->vecsz, 
-			 p->r != p->rio && p->r != p->iio, dp)) {
-	       if (p->r != p->rio && p->r != p->iio)
-		    return 1;  /* can always operate out-of-place */
+     const S *ego = (const S *) ego_;
+     const problem_rdft2 *p = (const problem_rdft2 *) p_;
+     if (FINITE_RNK(p->vecsz->rnk)
+	 && p->vecsz->rnk > 0
+	 && pickdim(ego, p->vecsz, 
+		    p->r != p->rio && p->r != p->iio, dp)) {
+	  if (p->r != p->rio && p->r != p->iio)
+	       return 1;  /* can always operate out-of-place */
 
-	       return(X(rdft2_inplace_strides)(p, *dp));
-	  }
+	  return(X(rdft2_inplace_strides)(p, *dp));
      }
 
      return 0;
 }
 
+
 static int applicable(const solver *ego_, const problem *p_,
 		      const planner *plnr, int *dp)
 {
@@ -140,7 +139,7 @@ static int applicable(const solver *ego_, const problem *p_,
 	     solvers. */
 	  if (p->sz->rnk == 0 && p->vecsz->rnk == 1) return 0;
 
-	  if (NONTHREADED_ICKYP(plnr)) 
+	  if (NO_NONTHREADEDP(plnr)) 
 	       return 0; /* prefer threaded version */
      }
 
@@ -155,7 +154,7 @@ static plan *mkplan(const solver *ego_, const problem *p_, planner *plnr)
      plan *cld;
      int vdim;
      iodim *d;
-     int ivs, ovs;
+     INT ivs, ovs;
 
      static const plan_adt padt = {
 	  X(rdft2_solve), awake, print, destroy
@@ -191,14 +190,16 @@ static plan *mkplan(const solver *ego_, const problem *p_, planner *plnr)
      X(ops_zero)(&pln->super.super.ops);
      pln->super.super.ops.other = 3.14159; /* magic to prefer codelet loops */
      X(ops_madd2)(pln->vl, &cld->ops, &pln->super.super.ops);
-     pln->super.super.pcost = pln->vl * cld->pcost;
+
+     if (p->sz->rnk != 1 || (p->sz->dims[0].n > 128))
+	  pln->super.super.pcost = pln->vl * cld->pcost;
 
      return &(pln->super.super);
 }
 
 static solver *mksolver(int vecloop_dim, const int *buddies, int nbuddies)
 {
-     static const solver_adt sadt = { mkplan };
+     static const solver_adt sadt = { PROBLEM_RDFT2, mkplan };
      S *slv = MKSOLVER(S, &sadt);
      slv->vecloop_dim = vecloop_dim;
      slv->buddies = buddies;
@@ -213,7 +214,7 @@ void X(rdft2_vrank_geq1_register)(planner *p)
      /* FIXME: Should we try other vecloop_dim values? */
      static const int buddies[] = { 1, -1 };
 
-     const int nbuddies = sizeof(buddies) / sizeof(buddies[0]);
+     const int nbuddies = (int)(sizeof(buddies) / sizeof(buddies[0]));
 
      for (i = 0; i < nbuddies; ++i)
           REGISTER_SOLVER(p, mksolver(buddies[i], buddies, nbuddies));
