@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003 Matteo Frigo
- * Copyright (c) 2003 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2006 Matteo Frigo
+ * Copyright (c) 2003, 2006 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *
  */
 
-/* $Id: rodft00e-r2hc.c,v 1.25 2003/04/08 20:48:08 stevenj Exp $ */
+/* $Id: rodft00e-r2hc.c,v 1.33 2006-01-27 02:10:50 athena Exp $ */
 
 /* Do a RODFT00 problem via an R2HC problem, with some pre/post-processing.
 
@@ -36,7 +36,9 @@
    similar numerical difficulties seems to be the direct adaptation of
    the Cooley-Tukey decomposition for antisymmetric data, but this
    would require a whole new set of codelets and it's not clear that
-   it's worth it at this point. */
+   it's worth it at this point.  However, we did implement the latter
+   algorithm for the specific case of odd n (logically adapting the
+   split-radix algorithm); see reodft00e-splitradix.c. */
 
 #include "reodft.h"
 
@@ -48,19 +50,19 @@ typedef struct {
      plan_rdft super;
      plan *cld;
      twid *td;
-     int is, os;
-     int n;
-     int vl;
-     int ivs, ovs;
+     INT is, os;
+     INT n;
+     INT vl;
+     INT ivs, ovs;
 } P;
 
 static void apply(const plan *ego_, R *I, R *O)
 {
      const P *ego = (const P *) ego_;
-     int is = ego->is, os = ego->os;
-     int i, n = ego->n;
-     int iv, vl = ego->vl;
-     int ivs = ego->ivs, ovs = ego->ovs;
+     INT is = ego->is, os = ego->os;
+     INT i, n = ego->n;
+     INT iv, vl = ego->vl;
+     INT ivs = ego->ivs, ovs = ego->ovs;
      R *W = ego->td->W;
      R *buf;
 
@@ -89,7 +91,7 @@ static void apply(const plan *ego_, R *I, R *O)
 	  /* FIXME: use recursive/cascade summation for better stability? */
 	  O[0] = buf[0] * 0.5;
 	  for (i = 1; i + i < n - 1; ++i) {
-	       int k = i + i;
+	       INT k = i + i;
 	       O[os * (k - 1)] = -buf[n - i];
 	       O[os * k] = O[os * (k - 2)] + buf[i];
 	  }
@@ -101,7 +103,7 @@ static void apply(const plan *ego_, R *I, R *O)
      X(ifree)(buf);
 }
 
-static void awake(plan *ego_, int flg)
+static void awake(plan *ego_, enum wakefulness wakefulness)
 {
      P *ego = (P *) ego_;
      static const tw_instr rodft00e_tw[] = {
@@ -109,9 +111,10 @@ static void awake(plan *ego_, int flg)
           { TW_NEXT, 1, 0 }
      };
 
-     AWAKE(ego->cld, flg);
+     X(plan_awake)(ego->cld, wakefulness);
 
-     X(twiddle_awake)(flg, &ego->td, rodft00e_tw, 2*ego->n, 1, (ego->n+1)/2);
+     X(twiddle_awake)(wakefulness,
+		      &ego->td, rodft00e_tw, 2*ego->n, 1, (ego->n+1)/2);
 }
 
 static void destroy(plan *ego_)
@@ -123,27 +126,24 @@ static void destroy(plan *ego_)
 static void print(const plan *ego_, printer *p)
 {
      const P *ego = (const P *) ego_;
-     p->print(p, "(rodft00e-r2hc-%d%v%(%p%))", ego->n - 1, ego->vl, ego->cld);
+     p->print(p, "(rodft00e-r2hc-%D%v%(%p%))", ego->n - 1, ego->vl, ego->cld);
 }
 
 static int applicable0(const solver *ego_, const problem *p_)
 {
+     const problem_rdft *p = (const problem_rdft *) p_;
      UNUSED(ego_);
-     if (RDFTP(p_)) {
-          const problem_rdft *p = (const problem_rdft *) p_;
-          return (1
-		  && p->sz->rnk == 1
-		  && p->vecsz->rnk <= 1
-		  && p->kind[0] == RODFT00
-	       );
-     }
 
-     return 0;
+     return (1
+	     && p->sz->rnk == 1
+	     && p->vecsz->rnk <= 1
+	     && p->kind[0] == RODFT00
+	  );
 }
 
 static int applicable(const solver *ego, const problem *p, const planner *plnr)
 {
-     return (!NO_UGLYP(plnr) && applicable0(ego, p));
+     return (!NO_SLOWP(plnr) && applicable0(ego, p));
 }
 
 static plan *mkplan(const solver *ego_, const problem *p_, planner *plnr)
@@ -152,7 +152,7 @@ static plan *mkplan(const solver *ego_, const problem *p_, planner *plnr)
      const problem_rdft *p;
      plan *cld;
      R *buf;
-     int n;
+     INT n;
      opcnt ops;
 
      static const plan_adt padt = {
@@ -201,7 +201,7 @@ static plan *mkplan(const solver *ego_, const problem *p_, planner *plnr)
 /* constructor */
 static solver *mksolver(void)
 {
-     static const solver_adt sadt = { mkplan };
+     static const solver_adt sadt = { PROBLEM_RDFT, mkplan };
      S *slv = MKSOLVER(S, &sadt);
      return &(slv->super);
 }
